package cn.iocoder.yudao.module.shipment.dal.mysql;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxMergePkgDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxOrderCheckInfo.BoxOrderCheckInfoDO;
import cn.iocoder.yudao.module.shipment.dto.BoxMergePkgPageDto;
import cn.iocoder.yudao.module.shipment.vo.box.BoxOrderInstallVO;
import cn.iocoder.yudao.module.shipment.vo.boxMergePkg.BoxMergePkgPDAListVO;
import cn.iocoder.yudao.module.shipment.vo.boxMergePkg.BoxMergePkgQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxMergePkg.BoxMergePkgWebVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;
import java.util.Set;

/**
 * 合包箱 Mapper
 * @author 江楠
 */
@Mapper
public interface BoxMergePkgMapper extends AbstractMapper<BoxMergePkgDO> {
    @Override
    default PageResult<BoxMergePkgDO> selectPage(PageVO page, Object object) {
        if (object instanceof BoxMergePkgQueryVO) {
            BoxMergePkgQueryVO vo = (BoxMergePkgQueryVO)object;
            return selectPage(page, new LambdaQuery<BoxMergePkgDO>()
                .eqIfPresent(BoxMergePkgDO::getPkgNum, vo.getPkgNum())
                .eqIfPresent(BoxMergePkgDO::getPkgLength, vo.getPkgLength())
                .eqIfPresent(BoxMergePkgDO::getPkgWidth, vo.getPkgWidth())
                .eqIfPresent(BoxMergePkgDO::getPkgHight, vo.getPkgHight())
                .eqIfPresent(BoxMergePkgDO::getStatus, vo.getStatus())
                .betweenIfPresent(BoxMergePkgDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(BoxMergePkgDO::getId));
        }

        return null;
    }

    @Override
    default List<BoxMergePkgDO> selectList(Object object) {
        if (object instanceof BoxMergePkgQueryVO) {
            BoxMergePkgQueryVO vo = (BoxMergePkgQueryVO)object;
            return selectList(new LambdaQuery<BoxMergePkgDO>()
                .eqIfPresent(BoxMergePkgDO::getPkgNum, vo.getPkgNum())
                .eqIfPresent(BoxMergePkgDO::getPkgLength, vo.getPkgLength())
                .eqIfPresent(BoxMergePkgDO::getPkgWidth, vo.getPkgWidth())
                .eqIfPresent(BoxMergePkgDO::getPkgHight, vo.getPkgHight())
                .eqIfPresent(BoxMergePkgDO::getStatus, vo.getStatus())
                .betweenIfPresent(BoxMergePkgDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(BoxMergePkgDO::getId));
        }

        return null;
    }

    @Select("<script>" +
            "select " +
            "t.id," +
            "t.pkg_num," +
            "t.pkg_length," +
            "t.pkg_width," +
            "t.pkg_hight," +
            "(select count(distinct a.order_id) from ecw_box_pkg_order a where a.pkg_id=t.id and a.deleted=0) as orderCount," +
            "ifnull(t.sum_num,(select sum(b.warehouse_in_info ->> '$.cartonsNum') from ecw_box_pkg_order a LEFT JOIN ecw_order_item b on a.order_item_id=b.order_item_id where a.pkg_id=t.id and a.deleted=0 and b.deleted=0)) as sumNum," +
            "ifnull(t.sum_volume,(select ROUND(sum(b.warehouse_in_info ->> '$.volume'),2) from ecw_box_pkg_order a LEFT JOIN ecw_order_item b on a.order_item_id=b.order_item_id where a.pkg_id=t.id and a.deleted=0 and b.deleted=0)) as sumVolume," +
            "ifnull(t.sum_weight,(select ROUND(sum(b.warehouse_in_info ->> '$.weight'),2) from ecw_box_pkg_order a LEFT JOIN ecw_order_item b on a.order_item_id=b.order_item_id where a.pkg_id=t.id and a.deleted=0 and b.deleted=0)) as sumWeight," +
            "(select sum(b.warehouse_in_info ->> '$.quantityAll') from ecw_box_pkg_order a LEFT JOIN ecw_order_item b on a.order_item_id=b.order_item_id where a.pkg_id=t.id and a.deleted=0 and b.deleted=0) as sumQuantity," +
            "t.`status`," +
            "t.create_time " +
            "from ecw_box_merge_pkg t " +
            "where t.shipment_id=#{query.shipmentId} and t.deleted=0 " +
            "<when test = 'query.status != null'>"+
            "and t.status=#{query.status} " +
            "</when>"+
            "</script>")
    IPage<BoxMergePkgPageDto> getBoxMergePkgPage(IPage<BoxMergePkgPageDto> mpPage, @Param("query")BoxMergePkgQueryVO query);

    @Select("<script>" +
            "select " +
            "t.id," +
            "t.pkg_num," +
            "t.pkg_length," +
            "t.pkg_width," +
            "t.pkg_hight," +
            "(select sum(b.warehouse_in_info ->> '$.cartonsNum') from ecw_box_pkg_order a LEFT JOIN ecw_order_item b on a.order_item_id=b.order_item_id where a.pkg_id=t.id and a.deleted=0 and b.deleted=0) as sumNum," +
            "(select ROUND(sum(b.warehouse_in_info ->> '$.volume'),2) from ecw_box_pkg_order a LEFT JOIN ecw_order_item b on a.order_item_id=b.order_item_id where a.pkg_id=t.id and a.deleted=0 and b.deleted=0) as sumVolume," +
            "(select ROUND(sum(b.warehouse_in_info ->> '$.weight'),2) from ecw_box_pkg_order a LEFT JOIN ecw_order_item b on a.order_item_id=b.order_item_id where a.pkg_id=t.id and a.deleted=0 and b.deleted=0) as sumWeight," +
            "t.`status`," +
            "t.create_time " +
            "from ecw_box_merge_pkg t " +
            "where t.id=#{id} and t.deleted=0" +
            "</script>")
    BoxMergePkgDO getMergePkgInfoById(@Param("id") Long id);

//    @Select("<script>" +
//            "select t.order_id,t.order_no,t.product_record,t.sum_num,t.sum_volume,t.sum_weight,t.sum_quantity,t1.tally_time ,t.customs_type " +
//            "from ecw_box_preload_goods t2 left join ecw_order t on t2.order_id=t.order_id " +
//            "LEFT JOIN ecw_box_tally t1 on t.order_id=t1.order_id and t1.deleted=0 and t1.shipment_id=#{shipmentId} " +
//            "where 1=1 and not exists (select 1 from ecw_box_pkg_order a where a.order_id=t.order_id and a.deleted=0) " +
//            "and t.deleted=0 and t2.deleted=0 and t.transport_id in (3,4) and t2.shipment_id=#{shipmentId} group by t2.order_id" +
//            "</script>")
@Select("<script>" +
        "select t.order_id,t.order_no,t.product_record,t.sum_num,t.sum_volume,t.sum_weight,t.sum_quantity,t1.tally_time ,t.customs_type " +
        "from ecw_box_preload_goods t2 left join ecw_order t on t2.order_id=t.order_id " +
        "LEFT JOIN ecw_box_tally t1 on t.order_id=t1.order_id and t1.deleted=0 and t1.shipment_id=#{shipmentId} " +
        "where 1=1   " +
        "<when test = 'unInstallOrderList != null'>"+
        "AND t2.order_id   in"+
        "<foreach item='orderid' index='index' collection='unInstallOrderList' open='(' separator=',' close=')'>"+
        "#{orderid}"+
        "</foreach> "+

        "</when>"+
        "and t.deleted=0 and t2.deleted=0 and t.transport_id in (3,4) and t2.shipment_id=#{shipmentId} group by t2.order_id" +
        "</script>")
    IPage<BoxMergePkgWebVO> getUnPkgPage(IPage<BoxMergePkgWebVO> mpPage,@Param("shipmentId") Long shipmentId,@Param("unInstallOrderList") List<Long> unInstallOrderList);


    @Select("<script>" +
            "select t.order_id,t.order_no,t.product_record,t.sum_num,t.sum_volume,t.sum_weight,t.sum_quantity,t1.tally_time ,t.customs_type " +
            "from ecw_box_preload_goods t2 left join ecw_order t on t2.order_id=t.order_id " +
            "LEFT JOIN ecw_box_tally t1 on t.order_id=t1.order_id and t1.deleted=0 and t1.shipment_id=#{shipmentId} " +
            "where 1=1   " +
            "<when test = 'installOrderList != null'>"+
            "AND t2.order_id  not in"+
            "<foreach item='orderid' index='index' collection='installOrderList' open='(' separator=',' close=')'>"+
            "#{orderid}"+
            "</foreach> "+

            "</when>"+
            "and t.deleted=0 and t2.deleted=0 and t.transport_id in (3,4) and t2.shipment_id=#{shipmentId} group by t2.order_id" +
            "</script>")
    IPage<BoxMergePkgWebVO> getUnPkgPage2(IPage<BoxMergePkgWebVO> mpPage,@Param("shipmentId") Long shipmentId,@Param("installOrderList") List<Long> installOrderList);


    @Select("<script>" +
            "select t.order_id,t.order_no,t.product_record,t.sum_num,t.sum_volume,t.sum_weight,t.sum_quantity,t1.tally_time, " +
            "(select count(*) from ecw_box_order_check_info b where b.shipment_id=#{query.shipmentId} and b.order_id=t2.order_id and b.type=1 and b.deleted=0) as installNum " +
            "from ecw_box_preload_goods t2 left join ecw_order t on t2.order_id=t.order_id " +
            "LEFT JOIN ecw_box_tally t1 on t.order_id=t1.order_id and t1.deleted=0 and t1.shipment_id=#{query.shipmentId} " +
            "where 1=1 and exists (select 1 from ecw_box_pkg_order a where a.order_id=t.order_id and a.deleted=0 and a.pkg_id=#{query.id}) " +
            "and t.deleted=0 and t.transport_id in (3,4) and t2.shipment_id=#{query.shipmentId} group by t2.order_id" +
            "</script>")
    IPage<BoxMergePkgWebVO> getPkgPageByPkgId(IPage page1,@Param("query") BoxMergePkgDO boxMergePkgDO);

    @Select("<script>" +
            "select " +
            "t.id," +
            "t.self_no," +
            "t.transport_type," +
            "t.shipment_status_air," +
            "t1.title_en," +
            "t.checkout_status " +
            "from ecw_box t " +
            "left JOIN ecw_warehouse t1 on t.dest_warehouse_id=t1.id " +
            "where t.transport_type in (3,4) " +
            "and t.ty_status=2112 " +
            "AND (t.start_warehouse_id in "+
            "<foreach item='id' index='index' collection='warehouseIdList' open='(' separator=',' close=')'>"+
            "#{id}"+
            "</foreach> or t.dest_warehouse_id in "+
            "<foreach item='id' index='index' collection='warehouseIdList' open='(' separator=',' close=')'>"+
            "#{id}"+
            "</foreach>)"+
            "<when test = 'selfNo != null'>"+
            "and t.self_no LIKE concat('%',#{selfNo},'%')"+
            "and t.air_shipment_status=171 " +
            "</when>"+
            "<when test = 'selfNo == null'>"+
            "and t.merge_pkg_status=161 " +
            "</when>"+
            "</script>")
    IPage<BoxMergePkgPDAListVO> getBoxMergePkgList4PDAPage(IPage<BoxMergePkgPDAListVO> mpPage, @Param("warehouseIdList") Set<Long> warehouseIds,@Param("selfNo") String selfNo);



    @Select("<script>" +
            "SELECT " +
            " ci.order_id as orderId, " +
            "ci.order_num_code as orderNumCode FROM " +
            " ecw_box_order_check_info  ci " +
            "LEFT JOIN  ecw_box_pkg_order pk ON pk.order_id = ci.order_id  AND ci.deleted = 0 " +
            "WHERE  ci.shipment_id = #{shipmentId} AND ci.deleted = 0 AND ci.type = 1 " +
            "<when test = 'orderId != null'>"+
            "and ci.order_id = #{orderId} " +
            "</when>"+
            "    GROUP BY ci.order_num_code " +
            "</script>")
    List<BoxOrderCheckInfoDO> getInstallBoxByShipmentId(@Param("shipmentId") Long shipmentId,@Param("orderId") Long orderId);
    @Select("<script>" +
            "SELECT\n" +
            " t.order_id,\n" +
            " t.order_no,\n" +
            " t.product_record,\n" +
            "\t t.sum_num,\n" +
            "\t t.sum_volume,\n" +
            "\t t.sum_weight,\n" +
            "\t t.sum_quantity,\n" +
            "\t t1.tally_time,\n" +
            "\t t.customs_type \n" +
            "FROM \n" +
            "\t ecw_box_preload_goods t2\n" +
            "\t LEFT JOIN ecw_order t ON t2.order_id = t.order_id\n" +
            "\t LEFT JOIN ecw_box_tally t1 ON t.order_id = t1.order_id \n" +
            "\t AND t1.deleted = 0 \n" +
            "\t AND t1.shipment_id = #{shipmentId} \n" +
            "\t\n" +
            "WHERE\n" +
            "\t 1 = 1 \n" +
            " \t AND  EXISTS ( SELECT 1 FROM ecw_box_pkg_order a WHERE a.order_id = t.order_id AND a.deleted = 0 ) \n" +
            "  AND t2.order_id  NOT  in ( " +
            "  SELECT\n" +
            "  ci.order_id \n" +
            "  FROM\n" +
            "  ecw_box_order_check_info  ci\n" +
            "  LEFT JOIN ecw_box_pkg_order pk ON pk.order_id = ci.order_id  AND ci.deleted = 0 \n" +
            "  WHERE ci.shipment_id = #{shipmentId} AND ci.deleted = 0 AND ci.type = 1    GROUP BY ci.order_num_code\n" +
            " ) " +
            "\t AND t.deleted = 0 \n" +
            "\t AND t2.deleted = 0 \n" +
            "\t AND t.transport_id IN ( 3, 4 ) \n" +
            "\t AND t2.shipment_id =  #{shipmentId} group by t2.order_id "+
            "</script>")
    List<BoxMergePkgWebVO> getNotInPkgOrderByShipmentId(@Param("shipmentId") Long shipmentId,@Param("orderIds")  List<Long> orderIds);
    @Select("<script>" +
            "select t2.order_id,o.order_no,  o.sum_num , case WHEN o.sum_num >1 THEN  1 ELSE 0 END as boxNumMoreOne from ecw_box_preload_goods t2 \n" +
            " LEFT JOIN ecw_order o ON o.order_id = t2.order_id AND o.deleted=0  \n" +
            " where 1=1  and t2.deleted=0  and t2.shipment_id=#{shipmentId}  group by t2.order_id " +
            "</script>")
    List<BoxOrderInstallVO> getBoxOrderInstallVo(Long shipmentId);

    @Select("<script>" +
            "SELECT\n" +
            "\t t.order_id,\n" +
            "\t t.order_no,\n" +
            "\t t.product_record,\n" +
            "\t t.sum_num,\n" +
            "\t t.sum_volume,\n" +
            "\t t.sum_weight,\n" +
            "\t t.sum_quantity,\n" +
            "\t t.customs_type \n" +
            "FROM \n" +
            "\tecw_box_preload_goods t2\n" +
            "LEFT JOIN ecw_order t ON t2.order_id = t.order_id\t\n" +
            "LEFT JOIN ecw_box_pkg_order a ON a.order_id = t2.order_id AND a.deleted = 0\n" +
            "WHERE 1=1 \n" +
            "AND t.deleted = 0 \n" +
            "\tAND t2.deleted = 0 \n" +
            "\tAND t.transport_id IN ( 3, 4 )  " +
            "\tAND t2.shipment_id =  #{shipmentId}   " +
            "<when test = 'orderIds != null'>"+
            "AND a.order_id   in"+
            "<foreach item='orderid' index='index' collection='orderIds' open='(' separator=',' close=')'>"+
            "#{orderid}"+
            "</foreach> "+
            "</when>"+
            " group by t2.order_id "+
            "</script>")
    List<BoxMergePkgWebVO> getAllBoxPgkOrder(@Param("shipmentId") Long shipmentId,@Param("orderIds") List<Long> orderIds);
    @Select("<script>" +
            "SELECT " +
            " ci.order_id as orderId, " +
            "ci.order_num_code as orderNumCode FROM " +
            " ecw_box_order_check_info  ci " +
            "LEFT JOIN  ecw_box_pkg_order pk ON pk.order_id = ci.order_id  AND ci.deleted = 0 " +
            "WHERE  ci.shipment_id = #{shipmentId} AND ci.deleted = 0 AND ci.type = 1 " +
            "<when test = 'orderIds != null'>"+
            "AND ci.order_id   in"+
            "<foreach item='orderid' index='index' collection='orderIds' open='(' separator=',' close=')'>"+
            " #{orderid} "+
            "</foreach> "+
            "</when>"+
            "  GROUP BY ci.order_num_code " +
            "</script>")
    List<BoxOrderCheckInfoDO> getAlllBoxOrderCheck(@Param("shipmentId") Long shipmentId,@Param("orderIds") List<Long> orderIds);
}
