package cn.iocoder.yudao.module.shipment.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentLoadExcelExportPushEvent;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentPreloadExcelExportPushEvent;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.number.NumberUtils;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.ecw.dal.dataobject.cabinet.CabinetDO;
import cn.iocoder.yudao.module.ecw.service.cabinet.CabinetService;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderGuanlian.OrderGuanlianDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderLocation.OrderLocationDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderWarehouseIn.OrderWarehouseInDO;
import cn.iocoder.yudao.module.order.dto.OrderBackInfoDto;
import cn.iocoder.yudao.module.order.service.order.*;
import cn.iocoder.yudao.module.order.service.orderWarehouseIn.OrderWarehouseInService;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackItemDo;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackVO;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrank.ProductBrankDO;
import cn.iocoder.yudao.module.product.service.productbrank.ProductBrankService;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dto.LoadDto;
import cn.iocoder.yudao.module.shipment.dto.PreloadDto;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.utils.CustomMergeStrategy;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.WriteTable;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.HAVE_NOT_PRELOAD_GOODS;


/**
 * 自编号订单excel导出监听
 *
 * @author zhengYi
 */
@Component("ShipmentLoadExcelExportListener")
@AllArgsConstructor
@Slf4j
public class ShipmentLoadExcelExportListener {

    private final OrderQueryService orderQueryService;
    private UeProperties ueProperties;
    private FileService fileService;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private BoxService boxService;

    @Resource
    private CabinetService cabinetService;

    @Resource
    private OrderItemService orderItemService;

    @Resource
    private ProductBrankService productBrankService;

    @Resource
    private BoxApprovalService boxApprovalService;

    @Resource
    private OrderGuanlianService orderGuanlianService;

    @Resource
    private OrderWarehouseInService orderWarehouseInService;

    @Resource
    private OrderService orderService;

//    @Value("${ue.templates-url}")
//    private String path;

    /**
     * 订单导出监听
     *
     * @param event 编号记录
     */
    @EventListener(ShipmentLoadExcelExportPushEvent.class)
    public void shipmentLoadExcelExportPushEvent(ShipmentLoadExcelExportPushEvent event) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/load/excel/");
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("shipment_load.xlsx");
                InputStream inputStream = null;
                inputStream = getClass().getClassLoader().getResourceAsStream(ueProperties.getTemplatesUrl() + "/load.xlsx");
                Map headMap = new HashMap();
                JSONObject jsonObject = JSONObject.parseObject(event.getRequestParams());
                Long shipmentId = jsonObject.getLong("shipmentId");
                List<BoxPreloadGoodsBackVO> goodsItemList = boxPreloadGoodsService.getShipGoodItemList(shipmentId);
                if (CollectionUtil.isEmpty(goodsItemList)) {
                    event.setResult(HAVE_NOT_PRELOAD_GOODS.getMsg());
                }
                BoxDO box = boxService.getBox(shipmentId);
                CabinetDO cabinet = cabinetService.getCabinet(box.getCabinetId());
                BoxApprovalDO boxApprovalDO = boxApprovalService.getOne(new LambdaQueryWrapperX<BoxApprovalDO>().eq(BoxApprovalDO::getShipmentId, shipmentId).eq(BoxApprovalDO::getApprovalType, 2).last("limit 1"));
                headMap.put("selfNo", box.getSelfNo());
                headMap.put("ctnNo",box.getCubNo());
                if(cabinet != null) {
                    headMap.put("cabinetName", cabinet.getName());
                }
                headMap.put("exportDate", DateUtils.formatDate(new Date()));
                if (boxApprovalDO != null) {
                    headMap.put("date", DateUtils.formatDate(boxApprovalDO.getCreateTime()));
                }
                File fileDir = new File(dir);
                if (!fileDir.exists()){
                    // 不存在则创建一个目录
                    fileDir.mkdirs();
                }
                ExcelWriter excelWriter = EasyExcel.write(dir + fileName).withTemplate(inputStream).build();
                WriteSheet sheet = EasyExcel.writerSheet(0).build();
                excelWriter.fill(headMap, sheet);
                if (box.getLdStatus() != null && box.getLdStatus()>=43) {//43、已装柜、待封柜
                    //以secId排序
                    goodsItemList.sort(Comparator.comparing(BoxPreloadGoodsBackVO::getSecId).thenComparing(BoxPreloadGoodsBackVO::getTidanNum).thenComparing(BoxPreloadGoodsBackVO::getId));
                    int allTotalNum = 0;
                    BigDecimal allTotalVolume = new BigDecimal(0);
                    BigDecimal allTotalWeight = new BigDecimal(0);
                    List<Long> secCount = goodsItemList.stream().map(BoxPreloadGoodsBackVO::getSecId).distinct().sorted().collect(Collectors.toList());
                    if (!CollectionUtils.isEmpty(secCount)) {
                        for (int i=0;i<secCount.size();i++){
                            List<String> list1 = new ArrayList();
                            List<List<String>> headList = new ArrayList<>();
                            String head = "";
                            if (i == 0) {
                                head = "第"+ NumberUtils.int2chineseNum(i+1) +"部分(Be loaded inside of the container)";
                            } else {
                                head = "第"+NumberUtils.int2chineseNum(i+1)+"部分(after "+ NumberUtils.format(i+1)+" parts)";
                            }
                            list1.add(head);
                            headList.add(list1);headList.add(list1);headList.add(list1);
                            headList.add(list1);headList.add(list1);headList.add(list1);
                            headList.add(list1);

                            headList.add(list1);headList.add(list1);headList.add(list1);
                            headList.add(list1);headList.add(list1);headList.add(list1);
                            headList.add(list1);

                            headList.add(list1);headList.add(list1);headList.add(list1);
                            //headList.add(list1);headList.add(list1);
                            //headList.add(list1);headList.add(list1);

                            List<LoadDto> list = new ArrayList<>();

                            int totalNum = 0;
                            BigDecimal totalVolume = new BigDecimal(0);
                            BigDecimal totalWeight = new BigDecimal(0);

                            Map<String,Integer> numMap = new HashMap();

                            Long part = secCount.get(i);

                            //以订单ID分组
                            Map<Long, List<BoxPreloadGoodsBackVO>> orderMap = goodsItemList.stream().filter(item -> item.getSecId().longValue()==part.longValue() && StringUtils.isNotBlank(item.getWarehouseInInfo()))
                                    .collect(Collectors.groupingBy(BoxPreloadGoodsBackVO::getOrderId,LinkedHashMap::new,Collectors.toList()));
                            WriteTable writeTable = EasyExcel.writerTable(i)
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap,null,null,0,true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap,null,null,1,true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap,null,null,2,true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap,null,null,7,true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap,null,null,10,true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap,null,null,12,true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap,null,null,13,true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap,null,null,14,true))
                                    .automaticMergeHead(true).head(headList).needHead(true).build();
                            for (Map.Entry<Long, List<BoxPreloadGoodsBackVO>> entry : orderMap.entrySet()) {
                                Long orderId = entry.getKey();
                                OrderBackInfoDto orderInfo = orderQueryService.info(orderId);
                                List<BoxPreloadGoodsBackVO> boxPreload = entry.getValue();
                                List<OrderItemDO> orderItems = boxPreload.stream().map(item -> orderItemService.getOrderItem(item.getOrderItemId())).collect(Collectors.toList());
                                for(OrderItemDO item : orderItems) {
                                    List<OrderWarehouseInDO> orderWarehouseInDOS = orderWarehouseInService.selectList(OrderWarehouseInDO::getOrderItemId, item.getOrderItemId());
                                    LoadDto preloadDto = new LoadDto();
                                    preloadDto.setOrderNo(orderInfo.getOrderNo());
                                    if (numMap.get(orderInfo.getOrderNo()) != null) {
                                        preloadDto.setId(String.valueOf(numMap.get(orderInfo.getOrderNo())));
                                    } else {
                                        preloadDto.setId(String.valueOf(numMap.size()+1));
                                        numMap.put(orderInfo.getOrderNo(),numMap.size()+1);
                                    }
                                    preloadDto.setMarks(orderInfo.getMarks());
                                    List<OrderGuanlianDO> guanlianList = orderGuanlianService.getGuanlianListByOrderId(orderInfo.getOrderId());
                                    //rIdList为此订单的所有关联订单
                                    List<Long> rIdList = new ArrayList<>();
                                    for (OrderGuanlianDO orderGuanlianDO : guanlianList) {
                                        Long oId = orderGuanlianDO.getOrderId();
                                        Long rId = orderGuanlianDO.getRelateOrderId();
                                        if(oId.equals(orderId)) {
                                            rIdList.add((rId));
                                        } else if(rId.equals(orderId)) {
                                            rIdList.add(oId);
                                        }
                                    }
                                    if (CollectionUtil.isNotEmpty(rIdList)) {
                                        List<OrderDO> orderDOS = orderService.selectList(OrderDO::getOrderId, rIdList);
                                        if (CollectionUtil.isNotEmpty(orderDOS)) {
                                            preloadDto.setRelateOrder(orderDOS.stream().map(OrderDO::getOrderNo).collect(Collectors.joining(",")));
                                        }
                                    }
                                    preloadDto.setProdTitleZh(item.getProdTitleZh()+ " " +item.getProdTitleEn());
                                    preloadDto.setMerge(orderInfo.getOldNumbers());
                                    ProductBrankDO productBrank = productBrankService.getProductBrank(item.getBrand());
                                    if(productBrank!=null && StringUtils.isNotBlank(productBrank.getTitleZh())){
                                        preloadDto.setBrand(productBrank.getTitleZh());
                                    }
                                    if(StringUtils.isNotBlank(item.getMaterial())){
                                        preloadDto.setMaterial(DictFrameworkUtils.getDictDataFromCache("product_material",item.getMaterial()+"").getLabel());
                                    }
                                    if(productBrank !=null && productBrank.getFiling()!=null){
                                        preloadDto.setProductRecord(productBrank.getFiling().equals("0")? "N":"Y");
                                    }
                                    preloadDto.setSumNum(item.getWarehouseInInfoVO().getCartonsNum());
                                    if(CollectionUtil.isNotEmpty(orderWarehouseInDOS)){
                                        String warehousingSpecificationType = orderWarehouseInDOS.stream().map(j -> {
                                            return DictFrameworkUtils.getDictDataFromCache("warehousing_specification_type", j.getSpecificationType() + "").getLabel();
                                        }).distinct().collect(Collectors.joining("/"));
                                        String packagingType = orderWarehouseInDOS.stream().map(j -> {
                                            return DictFrameworkUtils.getDictDataFromCache("packaging_type", j.getUnit()).getLabel();
                                        }).distinct().collect(Collectors.joining("/"));
                                        preloadDto.setUnit(warehousingSpecificationType);
                                        preloadDto.setPackageType(packagingType);
                                    }
                                    preloadDto.setSumVolume(item.getWarehouseInInfoVO().getVolume());

                                    preloadDto.setSumWeight(item.getWarehouseInInfoVO().getWeight());
                                    if(orderInfo.getCustomsType()!=null){
                                        preloadDto.setCustomsType(DictFrameworkUtils.getDictDataFromCache("customs_type",orderInfo.getCustomsType()+"").getLabel());
                                    }
                                    preloadDto.setWorth(item.getWorth());
                                    preloadDto.setQuantity(item.getWarehouseInInfoVO().getQuantityAll());
                                    list.add(preloadDto);
                                    totalNum = totalNum + item.getWarehouseInInfoVO().getCartonsNum();
                                    totalVolume = totalVolume.add(item.getWarehouseInInfoVO().getVolume());
                                    totalWeight = totalWeight.add(item.getWarehouseInInfoVO().getWeight());
                                }
                            }
                            LoadDto preloadDto = new LoadDto();
                            preloadDto.setId("TOTAL");
                            preloadDto.setSumNum(totalNum);
                            preloadDto.setSumVolume(totalVolume);
                            preloadDto.setSumWeight(totalWeight);
                            list.add(preloadDto);
                            allTotalNum = allTotalNum + totalNum;
                            allTotalVolume = allTotalVolume.add(totalVolume);
                            allTotalWeight = allTotalWeight.add(totalWeight);
                            if(i==secCount.size()-1){
                                LoadDto preloadDto1 = new LoadDto();
                                preloadDto1.setId("IN TOTAL");
                                preloadDto1.setSumNum(allTotalNum);
                                preloadDto1.setSumVolume(allTotalVolume);
                                preloadDto1.setSumWeight(allTotalWeight);
                                list.add(preloadDto1);
                            }
                            excelWriter.write(list,sheet,writeTable);
                        }
                    }

                }
                excelWriter.finish();
                inputStream.close();
                // 获取到临时文件
                File file = new File(dir + fileName);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());
            } catch (Exception e) {

                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }
}
