package cn.iocoder.yudao.module.shipment.listener;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentAirLoadExcelExportPushEvent;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentLoadExcelExportPushEvent;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentLoadExcelExportPushEvent2;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.framework.common.util.number.NumberUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.ecw.dal.dataobject.cabinet.CabinetDO;
import cn.iocoder.yudao.module.ecw.service.cabinet.CabinetService;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderGuanlian.OrderGuanlianDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderWarehouseIn.OrderWarehouseInDO;
import cn.iocoder.yudao.module.order.dto.OrderBackInfoDto;
import cn.iocoder.yudao.module.order.service.order.*;
import cn.iocoder.yudao.module.order.service.orderWarehouseIn.OrderWarehouseInService;
import cn.iocoder.yudao.module.order.vo.order.OrderWarehouseInDetailsVO;
import cn.iocoder.yudao.module.order.vo.orderItem.OrderItemBackVO;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackItemVO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrank.ProductBrankDO;
import cn.iocoder.yudao.module.product.service.product.ProductAttrService;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.product.service.productbrank.ProductBrankService;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxMergePkgDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPkgOrderDO;
import cn.iocoder.yudao.module.shipment.dto.AirLoadDto;
import cn.iocoder.yudao.module.shipment.dto.LoadDto;
import cn.iocoder.yudao.module.shipment.dto.PreloadDto;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxMergePkg.BoxMergePkgService;
import cn.iocoder.yudao.module.shipment.service.boxPkgOrder.BoxPkgOrderService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.utils.CustomAirLoadMergeStrategyNew;
import cn.iocoder.yudao.module.shipment.utils.CustomCellWriteHandler;
import cn.iocoder.yudao.module.shipment.utils.CustomMergeStrategy;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.WriteTable;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.HAVE_NOT_PRELOAD_GOODS;




/**
 * 自编号订单excel导出监听
 * 修改模板样式和添加列字段
 * 海运已装单数据导出
 * NG2408045L
 * @author zhengYi
 */
@Component("ShipmentLoadExcelExportListener2")
@AllArgsConstructor
@Slf4j
public class ShipmentLoadExcelExportListener2 {

    private final OrderQueryService orderQueryService;
    private UeProperties ueProperties;
    private FileService fileService;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private BoxService boxService;

    @Resource
    private CabinetService cabinetService;

    @Resource
    private OrderItemService orderItemService;

    @Resource
    private ProductBrankService productBrankService;

    @Resource
    private BoxApprovalService boxApprovalService;

    @Resource
    private OrderGuanlianService orderGuanlianService;

    @Resource
    private OrderWarehouseInService orderWarehouseInService;

    @Resource
    private OrderService orderService;

    @Resource
    private ProductAttrService productAttrService;

    private static List<Integer> colorList = new ArrayList<>();
    static {
        colorList.add(22);
        colorList.add(41);
        colorList.add(42);
        colorList.add(43);
        colorList.add(44);
        colorList.add(45);
        colorList.add(46);
        colorList.add(47);
    }

    /**
     * 订单导出监听
     *
     * @param event 编号记录
     */
    @EventListener(ShipmentLoadExcelExportPushEvent.class)
    public void shipmentLoadExcelExportPushEvent(
            ShipmentLoadExcelExportPushEvent event) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                DateTimeFormatter formatter =
                        DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/load/excel/");
                String fileName = event.getUserId().
                        toString().concat(StrUtil.DASHED).
                        concat(event.getUserType().toString()).
                        concat(StrUtil.DASHED).concat(nowTime).
                        concat("shipment_load.xlsx");

                InputStream inputStream = null;
                //获取模板 2024-06-05 添加注释
                inputStream = getClass().getClassLoader().
                        getResourceAsStream(
                                ueProperties.getTemplatesUrl() + "/load.xlsx");
                Map headMap = new HashMap();
                JSONObject jsonObject =
                        JSONObject.parseObject(event.getRequestParams());


                Long shipmentId = jsonObject.getLong("shipmentId");
                List<BoxPreloadGoodsBackVO> goodsItemList =
                        boxPreloadGoodsService.getShipGoodItemList(shipmentId);
                if (CollectionUtil.isEmpty(goodsItemList)) {
                    event.setResult(HAVE_NOT_PRELOAD_GOODS.getMsg());
                }

                BoxDO box = boxService.getBox(shipmentId);
                CabinetDO cabinet = cabinetService.getCabinet(box.getCabinetId());
                BoxApprovalDO boxApprovalDO =
                        boxApprovalService.getOne(
                                new LambdaQueryWrapperX<BoxApprovalDO>().
                                        eq(BoxApprovalDO::getShipmentId, shipmentId).
                                        eq(BoxApprovalDO::getApprovalType, 2).
                                        last("limit 1"));

                headMap.put("selfNo", box.getSelfNo());
                headMap.put("ctnNo", box.getCubNo());
                if (cabinet != null) {
                    headMap.put("cabinetName", cabinet.getName());
                }
                headMap.put("exportDate", DateUtils.formatDate(new Date()));
                if (boxApprovalDO != null) {
                    headMap.put("date",
                            DateUtils.formatDate(boxApprovalDO.getCreateTime()));
                }
                File fileDir = new File(dir);
                if (!fileDir.exists()) {
                    // 不存在则创建一个目录
                    fileDir.mkdirs();
                }
                //write 写入的文件路径
                //withTemplate 使用的模板
                ExcelWriter excelWriter =
                        EasyExcel.write(dir + fileName).
                                withTemplate(inputStream).build();

                WriteSheet sheet = EasyExcel.writerSheet(0).build();
                excelWriter.fill(headMap, sheet);

                //商品属性获取
                List<ProductAttrDO> ProductAttrList =
                        productAttrService.getProductAttrList();
                Map<Long, String> ProductAttrMap = null;
                if (CollectionUtil.isEmpty(ProductAttrList) == false) {
                    ProductAttrMap = ProductAttrList.stream().
                            collect(Collectors.toMap(ProductAttrDO::getId,
                                    ProductAttrDO::getAttrName));
                }
                //end 商品属性获取

                if (box.getLdStatus() != null && box.getLdStatus() >= 43) {
                    //43、已装柜、待封柜
                    //以secId排序
                    goodsItemList.sort(Comparator.comparing(
                                    BoxPreloadGoodsBackVO::getSecId).
                            thenComparing(BoxPreloadGoodsBackVO::getTidanNum).
                            thenComparing(BoxPreloadGoodsBackVO::getId));

                    int allTotalNum = 0;
                    BigDecimal allTotalVolume = new BigDecimal(0);
                    BigDecimal allTotalWeight = new BigDecimal(0);
                    List<Long> secCount =
                            goodsItemList.stream().
                                    map(BoxPreloadGoodsBackVO::getSecId).
                                    distinct().sorted().collect(Collectors.toList());

                    if (!CollectionUtils.isEmpty(secCount)) {
                        for (int i = 0; i < secCount.size(); i++) {
                            List<String> list1 = new ArrayList();
                            List<List<String>> headList = new ArrayList<>();
                            String head = "";
                            if (i == 0) {
                                head = "第" + NumberUtils.int2chineseNum(i + 1)
                                        + "部分(Be loaded inside of the container)";
                            } else {
                                head = "第" + NumberUtils.int2chineseNum(i + 1)
                                        + "部分(after " + NumberUtils.format(i + 1) + " parts)";
                            }
                            list1.add(head);

                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);

                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);

                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);
                            headList.add(list1);

                            List<LoadDto> list = new ArrayList<>();

                            int totalNum = 0;
                            BigDecimal totalVolume = new BigDecimal(0);
                            BigDecimal totalWeight = new BigDecimal(0);

                            Map<String, Integer> numMap = new HashMap();

                            Long part = secCount.get(i);

                            List<Integer> mergeCountList = new ArrayList();
                            List<Integer> colorRowList = new ArrayList<>();
                            AtomicInteger colorRowNum = new AtomicInteger(5);
                            Map<String,Integer> colorMap = new HashMap<>();
                            Map<String,Integer> mergePkgCountMap = new LinkedHashMap<>();
                            int colorIndex = 0;

                            //以订单ID分组
                            Map<Long, List<BoxPreloadGoodsBackVO>> orderMap =
                                    goodsItemList.stream().filter(item -> item.getSecId().longValue() ==
                                                    part.longValue() &&
                                                    StringUtils.isNotBlank(item.getWarehouseInInfo()))
                                            .collect(Collectors.groupingBy(BoxPreloadGoodsBackVO::getOrderId,
                                                    LinkedHashMap::new, Collectors.toList()));

                            for (Map.Entry<Long, List<BoxPreloadGoodsBackVO>> entry : orderMap.entrySet()) {
                                Long orderId = entry.getKey();
                                List<OrderWarehouseInBackItemVO> orderWarehouseInBackItemVOS =
                                        orderWarehouseInService.getOrderWarehouseInListOrderByInTime(orderId);

                                for (OrderWarehouseInBackItemVO item : orderWarehouseInBackItemVOS) {
                                    int andIncrement = colorRowNum.getAndIncrement();

                                    if (StringUtils.isNotBlank(item.getOrderWarehouseInDetails())) {
                                        JSONArray jsonArray = JSONArray.parseArray(item.getOrderWarehouseInDetails());
                                        if (jsonArray.size() > 0) {
                                            colorMap.put(String.valueOf(andIncrement),colorList.get(colorIndex));
                                            colorRowList.add(andIncrement);//这里加1是为了设置颜色时过滤掉入仓记录那条
                                        }
                                    }
                                    int mergeCount = 1;
                                    if (StringUtils.isNotBlank(item.getOrderWarehouseInDetails())) {
                                        item.setOrderWarehouseInDetailsVOList(JsonUtils.parseArray(item.getOrderWarehouseInDetails(), OrderWarehouseInDetailsVO.class));
                                    }
                                    if (CollectionUtil.isNotEmpty(item.getOrderWarehouseInDetailsVOList())) {
                                        mergeCount = mergeCount + item.getOrderWarehouseInDetailsVOList().size();
                                        List<OrderWarehouseInDetailsVO> orderWarehouseInDetailsVOList =
                                                item.getOrderWarehouseInDetailsVOList();
                                        for (int j = 0; j < orderWarehouseInDetailsVOList.size(); j++) {
                                            int andIncrement1 = colorRowNum.getAndIncrement();
                                            colorMap.put(String.valueOf(andIncrement1),colorList.get(colorIndex));
                                            colorRowList.add(andIncrement1);
                                        }
                                    }
                                    mergeCountList.add(mergeCount);
                                    if (colorIndex == 7) {
                                        colorIndex = 0;
                                    } else {
                                        colorIndex ++;
                                    }

                                }
                            }

                            //registerWriteHandler(new XXXStrategy(需要的参数))
                            //自定义单元格合并策略
                            WriteTable writeTable = EasyExcel.writerTable(i)
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap, null, null, 0, true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap, null, null, 1, true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap, null, null, 2, true))
                                    //2024-06-11 begin
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,3, colorMap))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,4, colorMap))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,5, colorMap))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,6, colorMap))
                                    .registerWriteHandler(new CustomCellWriteHandler(colorRowList,7, colorMap))
                                    //.registerWriteHandler(new CustomMergeStrategy(orderMap, null, null, 7, true)) //end
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap, null, null, 10, true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap, null, null, 12, true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap, null, null, 13, true))
                                    .registerWriteHandler(new CustomMergeStrategy(orderMap, null, null, 14, true))
                                    .automaticMergeHead(true).head(headList).needHead(true).build();


                            for (Map.Entry<Long, List<BoxPreloadGoodsBackVO>> entry : orderMap.entrySet()) {
                                Long orderId = entry.getKey();
                                OrderBackInfoDto orderInfo = orderQueryService.info(orderId);
                                List<BoxPreloadGoodsBackVO> boxPreload = entry.getValue();
                                List<OrderItemDO> orderItems =
                                        boxPreload.stream().map(item -> orderItemService.
                                                        getOrderItem(item.getOrderItemId())).
                                                collect(Collectors.toList());

                                for (OrderItemDO item : orderItems) {
                                    List<OrderWarehouseInDO> orderWarehouseInDOS =
                                            orderWarehouseInService.selectList(
                                                    OrderWarehouseInDO::getOrderItemId,
                                                    item.getOrderItemId());
                                    LoadDto preloadDto = new LoadDto();
                                    preloadDto.setOrderNo(orderInfo.getOrderNo());
                                    if (numMap.get(orderInfo.getOrderNo()) != null) {
                                        preloadDto.setId(String.valueOf(numMap.get(orderInfo.getOrderNo())));
                                    } else {
                                        preloadDto.setId(String.valueOf(numMap.size() + 1));
                                        numMap.put(orderInfo.getOrderNo(), numMap.size() + 1);
                                    }
                                    preloadDto.setMarks(orderInfo.getMarks());
                                    List<OrderGuanlianDO> guanlianList =
                                            orderGuanlianService.getGuanlianListByOrderId(
                                                    orderInfo.getOrderId());
                                    //rIdList为此订单的所有关联订单
                                    List<Long> rIdList = new ArrayList<>();
                                    for (OrderGuanlianDO orderGuanlianDO : guanlianList) {
                                        Long oId = orderGuanlianDO.getOrderId();
                                        Long rId = orderGuanlianDO.getRelateOrderId();
                                        if (oId.equals(orderId)) {
                                            rIdList.add((rId));
                                        } else if (rId.equals(orderId)) {
                                            rIdList.add(oId);
                                        }
                                    }
                                    if (CollectionUtil.isNotEmpty(rIdList)) {
                                        List<OrderDO> orderDOS =
                                                orderService.selectList(OrderDO::getOrderId, rIdList);
                                        if (CollectionUtil.isNotEmpty(orderDOS)) {
                                            preloadDto.setRelateOrder(orderDOS.stream().map(
                                                    OrderDO::getOrderNo).collect(Collectors.joining(",")));
                                        }
                                    }
                                    preloadDto.setProdTitleZh(item.getProdTitleZh() + " " + item.getProdTitleEn());
                                    preloadDto.setMerge(orderInfo.getOldNumbers());
                                    ProductBrankDO productBrank =
                                            productBrankService.getProductBrank(item.getBrand());
                                    if (productBrank != null && StringUtils.isNotBlank(productBrank.getTitleZh())) {
                                        preloadDto.setBrand(productBrank.getTitleZh());
                                    }
                                    //材质
                                    if (StringUtils.isNotBlank(item.getMaterial())) {
                                        preloadDto.setMaterial(
                                                DictFrameworkUtils.getDictDataFromCache("product_material",
                                                        item.getMaterial() + "").getLabel());
                                    }
                                    //品牌
                                    if (productBrank != null && productBrank.getFiling() != null) {
                                        preloadDto.setProductRecord(productBrank.getFiling().equals("0") ? "N" : "Y");
                                    }

                                    //用途
                                    if (item.getUsageIds() == null || item.getUsageIds().length() == 0) {
                                        //preloadDto.setUsageIds("");
                                    } else {
                                        //在缓存数据字典中获取用途信息
                                        String sUserdName = DictFrameworkUtils.getDictDataFromCache(
                                                "order_item_usage",
                                                item.getUsageIds() + "").getLabel();
                                        //preloadDto.setUsageIds(sUserdName);
                                    }

                                    //商品特性赋值
                                    String sR = getAttrName(item, ProductAttrMap);
                                    //preloadDto.setProdAttrIds(sR);

                                    preloadDto.setSumNum(item.getWarehouseInInfoVO().getCartonsNum());

                                    if (CollectionUtil.isNotEmpty(orderWarehouseInDOS)) {
                                        String warehousingSpecificationType = orderWarehouseInDOS.stream().map(j -> {
                                            return DictFrameworkUtils.getDictDataFromCache("warehousing_specification_type", j.getSpecificationType() + "").getLabel();
                                        }).distinct().collect(Collectors.joining("/"));
                                        String packagingType = orderWarehouseInDOS.stream().map(j -> {
                                            return DictFrameworkUtils.getDictDataFromCache("packaging_type", j.getUnit()).getLabel();
                                        }).distinct().collect(Collectors.joining("/"));
                                        preloadDto.setUnit(warehousingSpecificationType);
                                        preloadDto.setPackageType(packagingType);
                                    }
                                    preloadDto.setSumVolume(item.getWarehouseInInfoVO().getVolume());
                                    preloadDto.setSumWeight(item.getWarehouseInInfoVO().getWeight());
                                    if (orderInfo.getCustomsType() != null) {
                                        preloadDto.setCustomsType(
                                                DictFrameworkUtils.getDictDataFromCache("customs_type",
                                                        orderInfo.getCustomsType() + "").getLabel());
                                    }
                                    preloadDto.setWorth(item.getWorth());
                                    preloadDto.setQuantity(item.getWarehouseInInfoVO().getQuantityAll());
                                    list.add(preloadDto);
                                    totalNum = totalNum + item.getWarehouseInInfoVO().getCartonsNum();
                                    totalVolume = totalVolume.add(item.getWarehouseInInfoVO().getVolume());
                                    totalWeight = totalWeight.add(item.getWarehouseInInfoVO().getWeight());
                                }
                            }
                            LoadDto preloadDto = new LoadDto();
                            preloadDto.setId("TOTAL");
                            preloadDto.setSumNum(totalNum);
                            preloadDto.setSumVolume(totalVolume);
                            preloadDto.setSumWeight(totalWeight);
                            list.add(preloadDto);
                            allTotalNum = allTotalNum + totalNum;
                            allTotalVolume = allTotalVolume.add(totalVolume);
                            allTotalWeight = allTotalWeight.add(totalWeight);
                            if (i == secCount.size() - 1) {
                                LoadDto preloadDto1 = new LoadDto();
                                preloadDto1.setId("IN TOTAL");
                                preloadDto1.setSumNum(allTotalNum);
                                preloadDto1.setSumVolume(allTotalVolume);
                                preloadDto1.setSumWeight(allTotalWeight);
                                list.add(preloadDto1);
                            }
                            excelWriter.write(list, sheet, writeTable);
                        }
                    }

                }

                excelWriter.finish();
                inputStream.close();
                // 获取到临时文件
                File file = new File(dir + fileName);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());

            } catch (Exception e) {
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }

    /*
      添加商品属性转换函数
     */
    private String getAttrName(OrderItemDO item, Map<Long, String> ProductAttrMap) {
        String sReult = "";
        //添加商品特性
        if (item.getProdAttrIds() == null || item.getProdAttrIds().length() == 0) {
            return sReult;
        } else {
            if (ProductAttrMap == null) {
                return sReult;
            } else {
                String sAttrName = "";
                List<String> resultList = Arrays.asList(
                        item.getProdAttrIds().split(","));
                for (String sId : resultList
                ) {
                    Long l = Long.parseLong(sId);
                    if (sAttrName.length() != 0) {
                        sAttrName += "," + ProductAttrMap.get(l);
                    } else {
                        sAttrName += ProductAttrMap.get(l);
                    }
                }
                sReult = sAttrName;
            }
        }
        //end 添加商品特性
        return sReult;
    }
}

