package cn.iocoder.yudao.module.shipment.service.boxCost;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.ecw.dal.dataobject.supplier.SupplierDO;
import cn.iocoder.yudao.module.ecw.service.supplier.SupplierService;
import cn.iocoder.yudao.module.shipment.convert.BoxCostConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCostDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxCostMapper;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.vo.boxCost.BoxCostCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCost.BoxCostQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCost.BoxCostUpdateReqVO;
import cn.iocoder.yudao.module.wealth.service.payable.PayableService;
import cn.iocoder.yudao.module.wealth.vo.payable.PayableCreateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 费用登记 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class BoxCostServiceImpl extends AbstractService<BoxCostMapper, BoxCostDO> implements BoxCostService {

    @Resource
    private BoxCostMapper boxCostMapper;

    @Resource
    private PayableService payableService;
    @Resource
    private SupplierService supplierService;
    @Resource
    private BoxService boxService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxCost(BoxCostCreateReqVO createReqVO) {
        // 插入
        BoxCostDO boxCost = BoxCostConvert.INSTANCE.convert(createReqVO);
        boxCostMapper.insert(boxCost);
        // 返回

        //新增应付款
        Long supplierId = boxCost.getSupplierId();
        SupplierDO supplierDO = supplierService.getSupplier(supplierId);
        PayableCreateReqVO req = new PayableCreateReqVO();
        req.setSupplierId(supplierId);
        req.setSupplierName(supplierDO.getCompanyZh());
        req.setFeeType(Long.valueOf(boxCost.getCostType()));
        req.setTotalAmount(boxCost.getPrice());
        req.setCurrencyId(boxCost.getPriceUnit());
        req.setRemark(boxCost.getRemarks());
        req.setState(0);
        req.setSource("pc");
        req.setStepsId(Long.valueOf(boxCost.getOpStepType()));
        Long shipmentId = boxCost.getShipmentId();
        BoxDO boxDO = boxService.getBox(shipmentId);
        req.setContainerId(shipmentId);
        req.setContainerNo(boxDO.getCubNo());
        req.setPayableNo(boxDO.getSelfNo());
        req.setStartWarehouseId(boxDO.getStartWarehouseId());
        req.setDestWarehouseId(boxDO.getDestWarehouseId());
        req.setTransportId(Long.valueOf(boxDO.getTransportType()));
        req.setShippingFeeId(boxCost.getId());
        // 费用产生日期
        req.setFreecsdate(boxCost.getFreecsdate());
        payableService.createPayable(req);
        return boxCost.getId();
    }

    @Override
    public void updateBoxCost(BoxCostUpdateReqVO updateReqVO) {
        // 更新
        BoxCostDO updateObj = BoxCostConvert.INSTANCE.convert(updateReqVO);
        boxCostMapper.updateById(updateObj);
    }

    @Override
    public void deleteBoxCost(Long id) {
        // 删除
        boxCostMapper.deleteById(id);
    }

    @Override
    public BoxCostDO getBoxCost(Long id) {
        return boxCostMapper.selectById(id);
    }

    @Override
    public List<BoxCostDO> getBoxCostList(Collection<Long> ids) {
        return boxCostMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxCostDO> getBoxCostPage(BoxCostQueryVO query, PageVO page) {
        return boxCostMapper.selectPage(page, query);
    }

    @Override
    public List<BoxCostDO> getBoxCostList(BoxCostQueryVO query) {
        return boxCostMapper.selectList(query);
    }
}
