package cn.iocoder.yudao.module.shipment.service.boxMergePkg;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.qrcode.QRCode;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.DeptWarehouseDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.depository.vo.dept.DeptWarehouseBaseVO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.ecw.service.region.RegionService;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderObjective.OrderObjectiveDO;
import cn.iocoder.yudao.module.order.service.order.OrderItemService;
import cn.iocoder.yudao.module.order.service.order.OrderObjectiveService;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.order.service.order.impl.OrderPrintTagServiceImpl;
import cn.iocoder.yudao.module.shipment.convert.BoxMergePkgConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.*;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxOrderCheckInfo.BoxOrderCheckInfoDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMergePkgMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMergePkgNumRecordMapper;
import cn.iocoder.yudao.module.shipment.dto.BoxMergePkgPageDto;
import cn.iocoder.yudao.module.shipment.enums.AirShipmentStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.MergePkgStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.OrderShipmentLog;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.service.boxOrderCheckInfo.BoxOrderCheckInfoService;
import cn.iocoder.yudao.module.shipment.service.boxPkgOrder.BoxPkgOrderService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.vo.box.BoxGoodsStatistics;
import cn.iocoder.yudao.module.shipment.vo.box.BoxOrderInstallVO;
import cn.iocoder.yudao.module.shipment.vo.boxMergePkg.*;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.PageDTO;
import com.google.zxing.WriterException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.DEPT_NOT_FOUND;

/**
 * 合包箱 Service 实现类
 *
 * @author 江楠
 */
@Service
@Validated
public class BoxMergePkgServiceImpl extends AbstractService<BoxMergePkgMapper, BoxMergePkgDO> implements BoxMergePkgService {

    private static final String REDIS_KEY_NUM = "airNum";

    @Resource
    private BoxMergePkgMapper boxMergePkgMapper;

    @Resource
    private WarehouseService warehouseService;

    @Resource
    private BoxService boxService;

    @Resource
    private RegionService regionService;

    @Resource
    private BoxMergePkgNumRecordMapper boxMergePkgNumRecordMapper;

    @Resource
    private OrderService orderService;

    @Resource
    private OrderQueryService orderQueryService;

    @Resource
    private OrderItemService orderItemService;

    @Resource
    private BoxOpLogService boxOpLogService;

    @Resource
    @Lazy
    private BoxPkgOrderService boxPkgOrderService;

    @Resource
    private OrderObjectiveService orderObjectiveService;

    @Resource
    private BoxApprovalService boxApprovalService;

    @Resource
    @Lazy
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private FileApi fileApi;

    @Resource
    @Lazy
    private BoxOrderCheckInfoService boxOrderCheckInfoService;

    @Value("${yudao.web.admin-ui.url}")
    private String basePath;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createBoxMergePkg(BoxMergePkgCreateReqVO createReqVO) {
        BoxDO boxDO = boxService.getById(createReqVO.getShipmentId());
        //校验业务逻辑
        checkBoxApproval(createReqVO.getShipmentId());
        RegionDO region = regionService.getRegion(
                warehouseService.getWarehouse(boxDO.getDestWarehouseId()).getShi());

        String year = DateUtil.format(new Date(), "yy");
        DecimalFormat df = new DecimalFormat("0000");
        Integer num = createReqVO.getNum();
        if (region == null || StringUtils.isBlank(region.getLabelCode())) {
            throw exception(COUNTRY_LABEL_CODE_NOT_EXISTS);
        }
        String code = region.getLabelCode().substring(0, 1);
        BoxMergePkgNumRecordDO boxMergePkgNumRecordDO =
                boxMergePkgNumRecordMapper.selectOne(BoxMergePkgNumRecordDO::getDestCountryId, boxDO.getDestCountryId());
        if (boxMergePkgNumRecordDO == null) {
            boxMergePkgNumRecordDO = new BoxMergePkgNumRecordDO();
            boxMergePkgNumRecordDO.setDestCountryId(boxDO.getDestCountryId());
            boxMergePkgNumRecordDO.setDestWarehouseId(boxDO.getDestWarehouseId());
            boxMergePkgNumRecordDO.setNum(1);
            boxMergePkgNumRecordDO.setTransportType(boxDO.getTransportType());
            boxMergePkgNumRecordMapper.insert(boxMergePkgNumRecordDO);
        }
        // 插入
        for (int i = 0; i < num; i++) {
            if (boxMergePkgNumRecordDO.getNum().intValue() <= 9999) {
                createReqVO.setPkgNum(code + year +
                        df.format(boxMergePkgNumRecordDO.getNum()));
            } else {
                createReqVO.setPkgNum(code + year +
                        String.valueOf(boxMergePkgNumRecordDO.getNum()));
            }

            // 添加箱码是否重复的校验功能
            BoxMergePkgDO boxMergePkg =
                    BoxMergePkgConvert.INSTANCE.convert(createReqVO);
            if (isHavingBoxCod(boxMergePkg.getPkgNum()) == 0) {
                boxMergePkgMapper.insert(boxMergePkg);
            } else {
                String sBox = boxMergePkg.getPkgNum();
                boxMergePkg.setPkgNum(sBox + "F");
                boxMergePkgMapper.insert(boxMergePkg);
            }
            boxMergePkgNumRecordDO.setNum(boxMergePkgNumRecordDO.getNum() + 1);
        }
        boxMergePkgNumRecordDO.setUpdateTime(new Date());
        boxMergePkgNumRecordMapper.updateById(boxMergePkgNumRecordDO);
        // 返回
        return "success";
    }

    private long isHavingBoxCod(String sBoxCode) {
        LambdaQueryWrapperX q1 = new LambdaQueryWrapperX<BoxMergePkgDO>().
                eq(BoxMergePkgDO::getPkgNum, sBoxCode);
        Long lH = boxMergePkgMapper.selectCount(q1);
        return lH;
    }

    @Override
    public void updateBoxMergePkg(BoxMergePkgUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxMergePkgExists(updateReqVO.getId());
        checkBoxApproval(updateReqVO.getShipmentId());
        // 更新
        BoxMergePkgDO updateObj = BoxMergePkgConvert.INSTANCE.convert(updateReqVO);
        boxMergePkgMapper.updateById(updateObj);
    }

    @Override
    public void deleteBoxMergePkg(Long id) {
        // 校验存在
        this.validateBoxMergePkgExists(id);
        checkBoxApproval(boxMergePkgMapper.selectById(id).getShipmentId());
        // 删除
        boxMergePkgMapper.deleteById(id);
        boxOrderCheckInfoService.remove(new LambdaQueryWrapperX<BoxOrderCheckInfoDO>()
                .eq(BoxOrderCheckInfoDO::getPkgId, id)
        );
    }

    private void validateBoxMergePkgExists(Long id) {
        if (boxMergePkgMapper.selectById(id) == null) {
            throw exception(BOX_MERGE_PKG_NOT_EXISTS);
        }
    }

    @Override
    public BoxMergePkgDO getBoxMergePkg(Long id) {
        return boxMergePkgMapper.selectById(id);
    }

    @Override
    public List<BoxMergePkgDO> getBoxMergePkgList(Collection<Long> ids) {
        return boxMergePkgMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxMergePkgPageDto> getBoxMergePkgPage(BoxMergePkgQueryVO query, PageVO page) {
        page.setField("t.id");
        IPage<BoxMergePkgPageDto> mpPage = MyBatisUtils.buildPage(page);
        boxMergePkgMapper.getBoxMergePkgPage(mpPage, query);
        return PageResult.of(mpPage);
    }

    @Override
    public List<BoxMergePkgDO> getBoxMergePkgList(BoxMergePkgQueryVO query) {
        return boxMergePkgMapper.selectList(query);
    }

//    @Override
//    public PageResult<BoxMergePkgWebVO> getUnPkgPage(PageVO page,Long shipmentId) {
//        PageResult<BoxMergePkgWebVO> pageResult = new PageResult<>();
//        //全部要装的箱子
//        List<BoxOrderCheckInfoDO>  tagList = new ArrayList<>();
//        List<OrderDO> orderList = boxMergePkgMapper.getBoxOrderCheckByShipmentId(shipmentId) ;
//        List<BoxOrderCheckInfoDO> installBox = boxMergePkgMapper.getInstallBoxByShipmentId(shipmentId,null) ;
//        List<BoxMergePkgWebVO> installPkgList = boxMergePkgMapper.getNotInPkgOrderByShipmentId(shipmentId) ;
//        List<BoxOrderCheckInfoDO> installList = new ArrayList<>() ;
//
////        if(CollectionUtil.isNotEmpty(installBox)){
////            installList.addAll(installBox.stream().map(s-> s.getOrderId()).collect(Collectors.toList())) ;
////        }
//
//        if(CollectionUtil.isNotEmpty(installPkgList)){
//            installPkgList.forEach(o->{
//                OrderDO order = new OrderDO() ;
//                BeanUtils.copyProperties(o,order);
//                List<BoxOrderCheckInfoDO>   tags = boxOrderCheckInfoService.getOrderTagNumList(order);
//                installList.addAll(tags)  ;
//            });
//        }
//        if(CollectionUtil.isNotEmpty(orderList)){
//            orderList.forEach(o->{
//                OrderDO order = new OrderDO() ;
//                BeanUtils.copyProperties(o,order);
//                List<BoxOrderCheckInfoDO>   tags = boxOrderCheckInfoService.getOrderTagNumList(order);
//                tagList.addAll(tags)  ;
//            });
//        }
//        //全部实装
//        List<BoxOrderCheckInfoDO> allInstallList = new ArrayList<>() ;
//        allInstallList.addAll(installList);
//        allInstallList.addAll(installBox) ;
//        //还未装的箱子 ，以订单号加 "-" 区分的数据
//        List<BoxOrderCheckInfoDO> unInstall = new ArrayList<>() ;
//        if(CollectionUtil.isNotEmpty(allInstallList)){
//            List<BoxOrderCheckInfoDO>    collect = tagList.stream().filter(item -> allInstallList.stream().anyMatch(ins -> item.getOrderNumCode().equals(ins.getOrderNumCode()))) .collect(Collectors.toList());
//            unInstall.addAll(collect) ;
//        }
//
//        List<Long> unInstallOrderList = new ArrayList<>() ;
//        if(CollectionUtil.isNotEmpty(unInstall)){
//            unInstallOrderList =  unInstall.stream().map(s-> s.getOrderId()).collect(Collectors.toList());
//        }else{
//            unInstallOrderList=null ;
//        }
//        IPage page1 = new PageDTO(page.getPage(),page.getRows());
//        IPage<BoxMergePkgWebVO> boxMergePkgWebVOS = boxMergePkgMapper.getUnPkgPage(page1,shipmentId,unInstallOrderList);
//
//        getUnOrderInfoPageResult(pageResult, boxMergePkgWebVOS,shipmentId);
//        return pageResult;
//    }


    @Override
    public PageResult<BoxMergePkgWebVO> getUnPkgPage(PageVO page, Long shipmentId) {
        PageResult<BoxMergePkgWebVO> pageResult = new PageResult<>();
        //全部要装的箱子
        List<BoxOrderCheckInfoDO> tagList = new ArrayList<>();
        //全部要装箱的订单
        List<BoxOrderInstallVO> orderInstallVOList =
                boxMergePkgMapper.getBoxOrderInstallVo(shipmentId);
        List<Long> allOrderIds = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(orderInstallVOList)) {
            orderInstallVOList.forEach(o -> {
                OrderDO order = new OrderDO();
                BeanUtils.copyProperties(o, order);
                List<BoxOrderCheckInfoDO> tags =
                        boxOrderCheckInfoService.getOrderTagNumList(order);
                tagList.addAll(tags);
                allOrderIds.add(o.getOrderId());
            });
        }

        //一个箱子的只要判断在 ecw_box_pkg_order 表有数据就可以，多个实装箱子的，要在合包标签表ecw_box_order_check_info 判断还有哪个箱子没装

        //已加入装箱列表的有哪些
        List<BoxMergePkgWebVO> allBoxPgkOrder =
                boxMergePkgMapper.getAllBoxPgkOrder(shipmentId, allOrderIds);
        //已加入合包标签表的有哪些
        List<BoxOrderCheckInfoDO> allBoxOrderCheck =
                boxMergePkgMapper.getAlllBoxOrderCheck(shipmentId, allOrderIds);


        //剔除数据，留下 带”-“ 还未装的
        //加入已装箱列表但未加入合包标签表的
        List<BoxMergePkgWebVO> noInBoxOrderCheck = new ArrayList<>();

        List<Long> unInstallOrderList = new ArrayList<>();

        //已加入装箱列表不为空，提取不在已装箱列表的数据
        if (CollectionUtil.isNotEmpty(allBoxPgkOrder) && CollectionUtil.isNotEmpty(allBoxOrderCheck)) {
            List<BoxMergePkgWebVO> pkgWebVOS = allBoxPgkOrder.stream().filter(item -> !allBoxOrderCheck.stream().anyMatch(ins -> item.getOrderId().equals(ins.getOrderId()) || item.getOrderId() == ins.getOrderId())).collect(Collectors.toList());
            noInBoxOrderCheck.addAll(pkgWebVOS);
            List<BoxOrderCheckInfoDO> collect = tagList.stream().filter(item -> !allBoxOrderCheck.stream().anyMatch(ins -> item.getOrderNumCode().equals(ins.getOrderNumCode()))).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(collect)) {
                List<BoxOrderCheckInfoDO> checkInfoDOS = collect.stream().filter(item -> !pkgWebVOS.stream().anyMatch(ins -> item.getOrderId().equals(ins.getOrderId()) || item.getOrderId() == ins.getOrderId())).collect(Collectors.toList());

                List<Long> orderIdList = checkInfoDOS.stream().map(s -> s.getOrderId()).collect(Collectors.toList());
                unInstallOrderList.addAll(orderIdList);

            }

        } else if (CollectionUtil.isNotEmpty(allBoxPgkOrder) && CollectionUtil.isEmpty(allBoxOrderCheck)) {
            if (orderInstallVOList.size() == allBoxPgkOrder.size()) {
//                    List<Long> orderIdList = allBoxPgkOrder.stream().map(s->s.getOrderId()).collect(Collectors.toList());
//                    unInstallOrderList.addAll(orderIdList);

            } else {
                List<BoxOrderInstallVO> info = orderInstallVOList.stream().filter(item -> !allBoxPgkOrder.stream().anyMatch(ins -> item.getOrderId().equals(ins.getOrderId()) || item.getOrderId() == ins.getOrderId())).collect(Collectors.toList());
                List<Long> orderIdList = info.stream().map(s -> s.getOrderId()).collect(Collectors.toList());
                unInstallOrderList.addAll(orderIdList);
            }

        } else if (CollectionUtil.isEmpty(allBoxPgkOrder) && CollectionUtil.isEmpty(allBoxOrderCheck)) {
            List<BoxOrderInstallVO> info = orderInstallVOList.stream().filter(item -> !allBoxPgkOrder.stream().anyMatch(ins -> item.getOrderId().equals(ins.getOrderId()) || item.getOrderId() == ins.getOrderId())).collect(Collectors.toList());
            List<Long> orderIdList = info.stream().map(s -> s.getOrderId()).collect(Collectors.toList());
            unInstallOrderList.addAll(orderIdList);
        } else if (CollectionUtil.isEmpty(allBoxPgkOrder) && CollectionUtil.isNotEmpty(allBoxOrderCheck)) {
            throw exception(ORDER_NOT_IN_BOX);
        }

        IPage page1 = new PageDTO(page.getPage(), page.getRows());
        if (CollectionUtil.isNotEmpty(unInstallOrderList)) {//剩余未装箱订单
            IPage<BoxMergePkgWebVO> boxMergePkgWebVOS = boxMergePkgMapper.getUnPkgPage(page1, shipmentId, unInstallOrderList);

            getUnOrderInfoPageResult(pageResult, boxMergePkgWebVOS, shipmentId);
        } else {//没有剩余未装箱数据
            List<Long> orderIdList = orderInstallVOList.stream().map(s -> s.getOrderId()).collect(Collectors.toList());
            List<Long> installOrderList = new ArrayList<>();
            installOrderList.addAll(orderIdList);
            IPage<BoxMergePkgWebVO> boxMergePkgWebVOS = boxMergePkgMapper.getUnPkgPage2(page1, shipmentId, installOrderList);

            getUnOrderInfoPageResult(pageResult, boxMergePkgWebVOS, shipmentId);
        }

        return pageResult;
    }

    @Override
    public PageResult<BoxMergePkgWebVO> getPkgPageByPkgId(BoxMergePkgUpdateReqVO boxMergePkgUpdateReqVO, PageVO page) {
        PageResult<BoxMergePkgWebVO> pageResult = new PageResult<>();
        IPage page1 = new PageDTO(page.getPage(), page.getRows());
        Long pkgId = boxMergePkgUpdateReqVO.getId();
        BoxMergePkgDO boxMergePkgDO = boxMergePkgMapper.selectOne(BoxMergePkgDO::getId, pkgId);
        IPage<BoxMergePkgWebVO> boxMergePkgWebVOS = boxMergePkgMapper.getPkgPageByPkgId(page1, boxMergePkgDO);
        boxMergePkgWebVOS.getRecords().stream().forEach(item -> item.setPkgNum(boxMergePkgDO.getPkgNum()));
        getOrderInfoPageResult(pageResult, boxMergePkgWebVOS);
        return pageResult;
    }

    @Override
    public BoxMergePkgDO getMergePkgInfoById(Long id) {
        BoxMergePkgDO boxMergePkgDO = boxMergePkgMapper.selectById(id);
        if (boxMergePkgDO.getSumVolume() == null||
        boxMergePkgDO.getSumVolume().compareTo(new BigDecimal(0))==0) {
            //
            boxMergePkgDO = boxMergePkgMapper.getMergePkgInfoById(id);

            //根据长宽高计算箱的体积
            if (boxMergePkgDO.getPkgHight() == null) {
                boxMergePkgDO.setPkgHight(new BigDecimal(0));
            }
            if (boxMergePkgDO.getPkgWidth() == null) {
                boxMergePkgDO.setPkgWidth(new BigDecimal(0));
            }
            if(boxMergePkgDO.getPkgLength()==null)
            {
                boxMergePkgDO.setPkgLength(new BigDecimal(0));
            }

            BigDecimal v=
                    boxMergePkgDO.getPkgHight().multiply(new BigDecimal(0.01)).
                            multiply(boxMergePkgDO.getPkgLength()).
                            multiply(new BigDecimal(0.01)).multiply(boxMergePkgDO.getPkgWidth()
                            ).multiply(new BigDecimal(0.01));

            //四舍五入
            BigDecimal v2= v.setScale(1,BigDecimal.ROUND_HALF_UP);
            boxMergePkgDO.setSumVolume(v2);

        }
        return boxMergePkgDO;
    }

    @Override
    public void finishMergePkg(Long shipmentId) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if (boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        checkBoxApproval(shipmentId);
        List<BoxMergePkgDO> boxMergePkgDOS = boxMergePkgMapper.selectList(BoxMergePkgDO::getShipmentId, shipmentId);
        long count = boxMergePkgDOS.stream().filter(item -> item.getStatus() != 3).count();
        if (count > 0L) {
            throw exception(MERGE_PKG_STATUS_CAN_NOT_FINISH);
        }
        int mergePkgStatus = boxDO.getMergePkgStatus();
        MergePkgStatusEnum merge = MergePkgStatusEnum.MERGED;
        if (mergePkgStatus != merge.getStatus()) {
            boxDO.setMergePkgStatus(merge.getStatus());

            //修改订单状态
//            boxService.updateOrderStatusByShipmentId(shipmentId, OrderStatusEnum.SHIPMENT.getValue(),
//                    null, BoxAirStatusEnum.WAIT_SET_SHIPMENT.getStatus(),
//                    null, null);

            boxService.addOrderLogByShipmentId(shipmentId, OrderShipmentLog.MERGE_PKG, "");
        }
        boxService.updateById(boxDO);

        //插入操作日志
        boxOpLogService.createBoxOpLog(boxDO.getId(), merge.getStatus(), merge.getText(), merge.getTextEn());
    }

    @Override
    public BoxMergePkgTagVO getMergeTagById(Long id) {
        BoxMergePkgTagVO boxMergePkgTagVO = new BoxMergePkgTagVO();
        String partPath = "airTag/" + DateUtils.formatDate(new Date());
        BoxMergePkgDO boxMergePkgDO = boxMergePkgMapper.selectById(id);
        BoxDO box = boxService.getBox(boxMergePkgDO.getShipmentId());
        WarehouseDO startWarehouse = warehouseService.getWarehouse(box.getStartWarehouseId());
        WarehouseDO endWarehouse = warehouseService.getWarehouse(box.getDestWarehouseId());
        String start = startWarehouse.getQianzhui();
        boxMergePkgTagVO.setStartWarehouse(start);
        String end = endWarehouse.getQianzhui();
        boxMergePkgTagVO.setEndWarehouse(end);
        String type = OrderPrintTagServiceImpl.hashMap.get(box.getTransportType());
        boxMergePkgTagVO.setTransportName(type);
        String date = DateUtil.formatDate(new Date());
        boxMergePkgTagVO.setDate(date);
        boxMergePkgTagVO.setCompany("EC");
        boxMergePkgTagVO.setMergePkgNo(boxMergePkgDO.getPkgNum());
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            QRCode.generateQRCodeImage(boxMergePkgDO.getPkgNum(), 350, 350, outputStream);
            String url = fileApi.createFile(partPath + "/" + UUID.fastUUID().toString(), outputStream.toByteArray());
            boxMergePkgTagVO.setQrCodeUrl(url);
            //需求修改，不需要条码
//            String barcodeUrl = fileApi.createFile(BarcodeUtil.generateBarCode39("https://groupage.cn/", null, null, null, false, null));
//            boxMergePkgTagVO.setBarcodeUrl(barcodeUrl);
        } catch (WriterException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                outputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return boxMergePkgTagVO;
    }

    @Override
    public List<BoxMergePkgPDAVO> getMergePkgByPkgNum(String pkgNum) {
        List<BoxMergePkgPDAVO> boxMergePkgPDAVOS = new ArrayList<>();
        BoxMergePkgDO boxMergePkgDO = boxMergePkgMapper.selectOne(BoxMergePkgDO::getPkgNum, pkgNum);
        if (boxMergePkgDO == null) {
            throw exception(BOX_MERGE_PKG_NOT_EXISTS);
        }
        List<BoxPkgOrderDO> boxPkgOrderDOS =
                boxPkgOrderService.selectList(BoxPkgOrderDO::getPkgId,
                        boxMergePkgDO.getId());

        List<Long> orderIds = boxPkgOrderDOS.stream().map(BoxPkgOrderDO::getOrderId).distinct().collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(orderIds)) {
            List<OrderDO> orderDOS =
                    orderService.selectList(OrderDO::getOrderId, orderIds);
            orderDOS.stream().forEach(item -> {
                BoxMergePkgPDAVO boxMergePkgPDAVO = new BoxMergePkgPDAVO();
                boxMergePkgPDAVO.setOrderId(item.getOrderId());
                boxMergePkgPDAVO.setOrderNo(item.getOrderNo());
                boxMergePkgPDAVO.setMarks(item.getMarks());
                boxMergePkgPDAVO.setPackageType(item.getPackageType());
                OrderObjectiveDO orderObjectiveDO = orderObjectiveService.getByOrderId(item.getOrderId());
                JSONObject jsonObject = JSONObject.parseObject(orderObjectiveDO.getObjective());
                boxMergePkgPDAVO.setDestinationName(jsonObject.getString("titleZh"));
                DictDataRespDTO transportType = DictFrameworkUtils.getDictDataFromCache("transport_type", String.valueOf(item.getTransportId()));
                boxMergePkgPDAVO.setTransType(I18nMessage.getLang() == 0 ? transportType.getLabel() : transportType.getLabelEn());
                boxMergePkgPDAVOS.add(boxMergePkgPDAVO);
            });
        }
        return boxMergePkgPDAVOS;
    }

    @Override
    public PageResult<BoxMergePkgPDAListVO> getBoxMergePkgList4PDAPage(String selfNo, PageVO page) {
        Long deptId = SecurityFrameworkUtils.getLoginUser().getDeptId();
        DeptWarehouseBaseVO deptWarehouseBaseVO = new DeptWarehouseBaseVO();
        deptWarehouseBaseVO.setDeptId(deptId);
        List<DeptWarehouseDO> deptWarehouseDOS = warehouseService.deptList(deptWarehouseBaseVO);

        if (CollectionUtil.isEmpty(deptWarehouseDOS)) {
            throw exception(DEPT_NOT_FOUND);
        }

        IPage<BoxMergePkgPDAListVO> mpPage = MyBatisUtils.buildPage(page);

        Set<Long> warehouseIds = deptWarehouseDOS.stream()
                .map(DeptWarehouseDO::getWarehouseId)
                .collect(Collectors.toSet());

        IPage<BoxMergePkgPDAListVO> result = boxMergePkgMapper.getBoxMergePkgList4PDAPage(mpPage, warehouseIds, selfNo);
        result.getRecords().stream().forEach(item -> {
            BoxGoodsStatistics boxStatistics = boxPreloadGoodsService.getBoxStatistics(item.getId());
            BeanUtils.copyProperties(boxStatistics, item);
        });
        return PageResult.of(result);
    }

    @Override
    public void closeMergePkg(BoxMergePkgUpdateReqVO updateReqVO) {
        boxService.checkOrderTagNum(updateReqVO.getId(), 1);
        this.updateBoxMergePkg(updateReqVO);
    }

    private void getOrderInfoPageResult(PageResult<BoxMergePkgWebVO> pageResult, IPage<BoxMergePkgWebVO> boxMergePkgWebVOIPage) {
        if (CollectionUtil.isNotEmpty(boxMergePkgWebVOIPage.getRecords())) {
            boxMergePkgWebVOIPage.getRecords().stream().forEach(item -> {
                List<OrderItemDO> orderItemDOS = orderItemService.selectList(new LambdaQueryWrapperX<OrderItemDO>()
                        .eq(OrderItemDO::getOrderId, item.getOrderId())
                        .isNotNull(OrderItemDO::getWarehouseInInfo)
                );
                item.setOrderItemDOS(orderItemDOS);
            });
            pageResult.setList(boxMergePkgWebVOIPage.getRecords());
        }
        pageResult.setPage(boxMergePkgWebVOIPage.getCurrent());
        pageResult.setPages(boxMergePkgWebVOIPage.getPages());
        pageResult.setRows(boxMergePkgWebVOIPage.getSize());
        pageResult.setTotal(boxMergePkgWebVOIPage.getTotal());
    }

    private void getUnOrderInfoPageResult(PageResult<BoxMergePkgWebVO> pageResult, IPage<BoxMergePkgWebVO> boxMergePkgWebVOIPage, Long shipmentId) {
        if (CollectionUtil.isNotEmpty(boxMergePkgWebVOIPage.getRecords())) {
            List<Long> orderIdList = boxMergePkgWebVOIPage.getRecords().stream().map(o -> o.getOrderId()).collect(Collectors.toList());
            List<OrderItemDO> AllOrderItemDOS = orderItemService.selectList(new LambdaQueryWrapperX<OrderItemDO>()
                    .in(OrderItemDO::getOrderId, orderIdList)
                    .isNotNull(OrderItemDO::getWarehouseInInfo)
            );
            boxMergePkgWebVOIPage.getRecords().stream().forEach(item -> {
//                List<OrderItemDO> orderItemDOS = orderItemService.selectList(new LambdaQueryWrapperX<OrderItemDO>()
//                        .eq(OrderItemDO::getOrderId, item.getOrderId())
//                        .isNotNull(OrderItemDO::getWarehouseInInfo)
//                );

                List<OrderItemDO> orderItemDOS = AllOrderItemDOS.stream().filter(all -> all.getOrderId() == item.getOrderId() || all.getOrderId().equals(item.getOrderId())).collect(Collectors.toList());

                List<BoxOrderCheckInfoDO> installBox = boxMergePkgMapper.getInstallBoxByShipmentId(shipmentId, item.getOrderId());

                item.setInstallNum(CollectionUtil.isNotEmpty(installBox) ? installBox.size() : 0);
                item.setOrderItemDOS(orderItemDOS);
            });
            pageResult.setList(boxMergePkgWebVOIPage.getRecords());
        }
        pageResult.setPage(boxMergePkgWebVOIPage.getCurrent());
        pageResult.setPages(boxMergePkgWebVOIPage.getPages());
        pageResult.setRows(boxMergePkgWebVOIPage.getSize());
        pageResult.setTotal(boxMergePkgWebVOIPage.getTotal());
    }

    private void checkBoxApproval(Long shipmentId) {
        BoxDO box = boxService.getBox(shipmentId);
        if (box.getAirShipmentStatus() != AirShipmentStatusEnum.UNSHIPMENT.getStatus()) {
            throw exception(MERGE_PKG_IS_FINISH);
        }
        Long aLong = boxApprovalService.selectCount(new LambdaQueryWrapperX<BoxApprovalDO>().eq(BoxApprovalDO::getShipmentId, shipmentId).eq(BoxApprovalDO::getApprovalStatus, 0));
        if (aLong != null && aLong > 0L) {
            throw exception(BOX_APPROVAL_EXIST);
        }
    }
}
