package cn.iocoder.yudao.module.shipment.service.boxOrderCheckInfo;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxOrderCheckInfo.BoxOrderCheckInfoDO;
import cn.iocoder.yudao.module.shipment.dto.OrderCheckInfoDto;
import cn.iocoder.yudao.module.shipment.vo.box.BoxOrderSearchVO;
import cn.iocoder.yudao.module.shipment.vo.boxLoadInfo.BoxLoadInfoOrderVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderCheckInfo.BoxOrderCheckInfoCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderCheckInfo.BoxOrderCheckInfoQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderCheckInfo.BoxOrderCheckInfoRemoveReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderCheckInfo.BoxOrderCheckInfoUpdateReqVO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 理货和合包标签 Service 接口
 *
 * @author 江楠
 */
public interface BoxOrderCheckInfoService extends IService<BoxOrderCheckInfoDO> {

    /**
     * 创建理货和合包标签
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBoxOrderCheckInfo(@Valid BoxOrderCheckInfoCreateReqVO createReqVO);

    /**
     * 更新理货和合包标签
     * @param updateReqVO 更新信息
     */
    void updateBoxOrderCheckInfo(@Valid BoxOrderCheckInfoUpdateReqVO updateReqVO);

    /**
     * 删除理货和合包标签
     * @param id 编号
     */
    void deleteBoxOrderCheckInfo(Long id);

    /**
     * 获得理货和合包标签
     * @param id 编号
     * @return 理货和合包标签
     */
    BoxOrderCheckInfoDO getBoxOrderCheckInfo(Long id);

    /**
     * 获得理货和合包标签列表
     * @param ids 编号
     * @return 理货和合包标签列表
     */
    List<BoxOrderCheckInfoDO> getBoxOrderCheckInfoList(Collection<Long> ids);

    /**
     * 获得理货和合包标签分页
     * @param page 分页查询
     * @param query 查询
     * @return 理货和合包标签分页
     */
    PageResult<BoxOrderCheckInfoDO> getBoxOrderCheckInfoPage(BoxOrderCheckInfoQueryVO query, PageVO page);

    /**
     * 获得理货和合包标签列表, 用于 Excel 导出
     * @param query 查询
     * @return 理货和合包标签列表
     */
    List<BoxOrderCheckInfoDO> getBoxOrderCheckInfoList(BoxOrderCheckInfoQueryVO query);

    Boolean batchCreate(BoxOrderCheckInfoCreateReqVO createReqVO);

    void deleteBoxLoadInfo(BoxOrderCheckInfoCreateReqVO createReqVO);

    CommonResult<BoxLoadInfoOrderVO> orderTagList(Long orderId,Integer type);

    OrderCheckInfoDto searchLoadOrderByBoxNo(BoxOrderSearchVO boxOrderSearchVO);

    void batchRemove(BoxOrderCheckInfoRemoveReqVO removeReqVO);

    void orderTally(Collection<Long> orderIdList, Long shipmentId);

    List<String> getOrderTagList(OrderDO orderDO) ;

    List<BoxOrderCheckInfoDO> getOrderTagNumList(OrderDO order);
}
